// ignore_for_file: curly_braces_in_flow_control_structures, prefer_is_empty

import 'dart:async';
import 'dart:developer';

import 'package:hub/backend/api_requests/api_calls.dart';
import 'package:hub/shared/components/molecules/modules/index.dart';
import 'package:hub/shared/helpers/database/database_helper.dart';

import 'package:rxdart/rxdart.dart';
import 'package:sqflite/sqflite.dart';

abstract class LicenseRemoteDataSource {
  Future<bool> fetchLicenses(bool isNewVersion);
  Future<void> cleanLicense();
  Future<void> processLicense();
  Future<void> setupLicense(Database database, bool isNewVersion);
}

class LicenseRemoteDataSourceImpl implements LicenseRemoteDataSource {
  static final LicenseRemoteDataSourceImpl _instance = LicenseRemoteDataSourceImpl._internal();
  factory LicenseRemoteDataSourceImpl() => _instance;
  LicenseRemoteDataSourceImpl._internal();

  
  final StreamController<void> _completerController = StreamController<void>.broadcast();
  Stream<void> get streamCompleterController => _completerController.stream;


  @override
  Future<void> processLicense() async {}

  @override
  Future<void> cleanLicense() async {
    LicenseRepositoryImpl.license.add(false);
    
  }

 @override
  Future<void> setupLicense(Database database, bool isNewVersion) async {
    final License license = License.getLicense(isNewVersion);

    final List<String> inactiveModuleKey = license.modules
        .where((module) => module.display == ModuleStatus.inactive.key)
        .map((module) => module.key)
        .toList();

    final List<String> activeModuleKey = license.modules
        .where((module) => module.display == ModuleStatus.active.key)
        .map((module) => module.key)
        .toList();

    final List<String> disabledModuleKey = license.modules
        .where((module) => module.display == ModuleStatus.disabled.key)
        .map((module) => module.key)
        .toList();

    await LicenseLocalDataSourceImpl().setByKey(inactiveModuleKey, 'INVISIVEL');
    await LicenseLocalDataSourceImpl().setByKey(activeModuleKey, 'VISIVEL');
    if (isNewVersion == true) {
      await LicenseLocalDataSourceImpl().setByKey(disabledModuleKey, 'VISIVEL');
      await LicenseLocalDataSourceImpl().setByKey(['FRE-HUB-PROPERTY'], 'VISIVEL');
    } else {
      await LicenseLocalDataSourceImpl().setByKey(disabledModuleKey, 'DESABILITADO');
    }

    LicenseRepositoryImpl.license.add(true);
  }

  @override
  Future<bool> fetchLicenses(bool isNewVersion) async {
    log('(A) => license');
    try {
      log('Obtendo licenças...');
      final response = await PhpGroup.getLicense();
      final dynamic responseBody = response.jsonBody;
      log('Licenças obtidas: $responseBody');

      if (response.jsonBody is! List<dynamic>) {
        late final String error;
        if (response.jsonBody is Map<String, dynamic>)
          error = response.jsonBody['error_msg'];
        else
          error = 'Erro desconhecido';

        throw Exception('Erro ao consultar licenças: $error');
      }

      if (responseBody == []) {
        await setupLicense(DatabaseStorage.database, isNewVersion);
        LicenseRepositoryImpl.license.add(false);
        return false;
      }

      List<Future<void>> saveOperations = [];
      for (var element in responseBody) {
        if (licenseContainsKey(element['key'])) {
          saveOperations.add(_saveModule(element));
        }
      }
      await Future.wait(saveOperations);

      log('() => License stream add');
      LicenseRepositoryImpl.license.add(true);
      _completerController.add(null);
      
      return true;
    } catch (e) {
      log('Erro ao obter licenças: $e');
      await setupLicense(DatabaseStorage.database, isNewVersion);
      return false;
    }
  }

  Future<void> waitForSaveCompletion() async {
    await LicenseRepositoryImpl.stream;
  }

  static bool licenseContainsKey(final String key) {
    return License.mods.contains(key);
  }

  static Future<void> _saveModule(final dynamic body) async {
    if (body is! Map<String, dynamic>) return;
    log('Salvando módulo: ${body.toString()}');
    await LicenseLocalDataSourceImpl().s(body['key'], body);
  }
}
