import 'package:hub/shared/components/molecules/modules/data/data_sources/license_local_data_source.dart';
import 'package:hub/shared/components/molecules/modules/data/repositories/license_repository_impl.dart';
import 'package:hub/shared/helpers/storage/keychain_storage.dart';
import 'package:sqflite/sqflite.dart';
import 'package:path/path.dart';
import 'dart:developer';

class DatabaseStorage {
  static final String _dbName = 'database.db';
  static final int _dbVersion = 3;

  static final DatabaseStorage instance = DatabaseStorage._internal();
  factory DatabaseStorage() => instance;
  DatabaseStorage._internal();

  static Database? _database;
  static Database get database {
    if (_database == null) {
      throw Exception('Database has not been initialized. Call init() first.');
    }
    return _database!;
  }

  bool isInitialized = false;

  Future<void> init() async {
    if (isInitialized) return;
    _database = await openDatabase(
      join(await getDatabasesPath(), _dbName),
      version: _dbVersion,
      onCreate: _onCreate,
      onUpgrade: _onUpgrade,
      onDowngrade: _onDowngrade,
    );
    await LicenseRepositoryImpl().setupLicense(false);
    isInitialized = true;
  }

  Future<void> _onCreate(Database database, int version) async {
    log('Creating database...');
    await database.execute(KeychainHelper.createKeychainTable);
    await database.execute(LicenseLocalDataSourceImpl.createLicenseTable);
    if (version >= 2) {
      await database.execute(LicenseLocalDataSourceImpl.updatePetsHistoryTrigger);
    }
    if (version >= 3) {
      await database.execute(LicenseLocalDataSourceImpl.updateDisplayTrigger);
    }
  }

  Future<void> _onUpgrade(Database database, int oldVersion, int newVersion) async {
    log('Upgrading database from version $oldVersion to $newVersion...');
    if (oldVersion < 2 && newVersion >= 2) {
      await database.execute(LicenseLocalDataSourceImpl.updatePetsHistoryTrigger);
    }
    if (oldVersion < 3 && newVersion >= 3) {
      await database.execute(LicenseLocalDataSourceImpl.updateDisplayTrigger);
    }
  }

  Future<void> _onDowngrade(Database database, int oldVersion, int newVersion) async {
    log('Downgrading database from version $oldVersion to $newVersion...');
    if (oldVersion >= 3 && newVersion < 3) {
      await database.execute(LicenseLocalDataSourceImpl.dropDisplayTrigger);
    }
    if (oldVersion >= 2 && newVersion < 2) {
      await database.execute(LicenseLocalDataSourceImpl.dropPetsHistoryTrigger);
    }
  }
}
